#! /usr/bin/env python

import os
import subprocess
from . import utils
import annot_utils.exon


def expression_main(args):

    input_bam = args.bam_file
    output_prefix = args.output_prefix
    mapq_thres = args.q
    keep_improper_pair = args.keep_improper_pair

    output_prefix_dir = os.path.dirname(output_prefix)
    if output_prefix_dir != "" and not os.path.exists(output_prefix_dir):
       os.makedirs(output_prefix_dir)

    annot_utils.exon.make_exon_info(output_prefix + ".refExon.bed.gz", "refseq", args.genome_id, args.grc, True)

    if not args.skip_bam_filt:    
    # utils.filterImproper(input_bam, output_prefix + ".filt.bam", mapq_thres, keep_improper_pair)
        utils.filterImproper(input_bam, output_prefix + ".filt.bam", keep_improper_pair) 


    intersect_abam = input_bam if args.skip_bam_filt else output_prefix + ".filt.bam"
    hout = open(output_prefix + ".exon.bed", 'w')
    subprocess.check_call(["bedtools", "intersect", "-a", intersect_abam,
                     "-b", output_prefix + ".refExon.bed.gz", "-wao", "-bed", "-split"], stdout = hout)
    hout.close()                

 
    utils.exon_base_count(output_prefix + ".exon.bed", output_prefix + ".exon2base.txt", mapq_thres)

    utils.mapped_base_count(output_prefix + ".exon.bed", output_prefix + ".mapped_base_count.txt", mapq_thres)

    utils.ref_base_count(output_prefix + ".exon2base.txt", output_prefix + ".ref2base.txt", output_prefix + ".refExon.bed.gz")

    utils.sym_base_count(output_prefix + ".ref2base.txt", output_prefix + ".sym2base.txt")

    utils.sym_fkpm(output_prefix + ".sym2base.txt", output_prefix + ".sym2fpkm.txt", output_prefix + ".mapped_base_count.txt")

    if not args.debug:
        subprocess.check_call(["rm", "-rf", output_prefix + ".refExon.bed.gz"])
        subprocess.check_call(["rm", "-rf", output_prefix + ".refExon.bed.gz.tbi"])
        subprocess.check_call(["rm", "-rf", output_prefix + ".filt.bam"])
        subprocess.check_call(["rm", "-rf", output_prefix + ".filt.bed12"])
        subprocess.check_call(["rm", "-rf", output_prefix + ".exon.bed"])
        subprocess.check_call(["rm", "-rf", output_prefix + ".exon2base.txt"])




#! /usr/bin/env python

from .run import *
import argparse

def create_parser():

    parser = argparse.ArgumentParser(prog = "genomon_expression")

    parser.add_argument("--version", action = "version", version = "genomon_expression-0.5.1")

    parser.add_argument("bam_file", metavar = "sequence.bam", default = None, type = str,
                        help = "the path to the bam file")

    parser.add_argument("output_prefix", metavar = "output_prefix", default = None, type = str, 
                        help = "the prefix of the output")

    parser.add_argument("--grc", default = False, action = 'store_true',
                        help = "convert chromosome names to Genome Reference Consortium nomenclature (default: %(default)s)")

    parser.add_argument("--genome_id", choices = ["hg19", "hg38", "mm10"], default = "hg19",
                        help = "the genome id used for selecting UCSC-GRC chromosome name corresponding files (default: %(default)s)")

    parser.add_argument("-q", metavar = "mapping_qual_thres", default='20', type=int,
                        help = "threshold for mapping quality for calculating base counts")

    parser.add_argument("--keep_improper_pair", action = 'store_true', default = False,
                        help = "keep improper paired reads (activate for single end reads)")

    parser.add_argument("--skip_bam_filt", action = 'store_true', default = False,
                        help = "Skip the bam filtering step where improer pairs, duplicated reads and supplementary reads are removed (default: %(default)s)")


    parser.add_argument("--debug", default = False, action = 'store_true', help = "keep intermediate files")

    return parser





#!/usr/bin/env python

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name = 'genomon_expression',
    version = '0.5.1',
    description='Python tools for calculating fkpm values from RNA-seq',
    url = 'https://github.com/Genomon-Project/GenomonExpression.git',
    author = 'Yuichi Shiraishi',
    author_email = 'friend1ws@gamil.com',
    license = 'GPLv3',

    classifiers = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Scientific/Engineering :: Bio-Informatics'
    ],

    packages = find_packages(exclude = ['tests']),
    install_requires = [],

    entry_points = {'console_scripts': ['genomon_expression = genomon_expression:main']}

)





#! /usr/bin/env python

from __future__ import print_function
import re, gzip
import pysam

re_refID =re.compile(r'\((N[MR]_\d+)\)')


def filterImproper(input_bam, output_bam, keep_improper_pair):
# def filterImproper(input_bam, output_bam, mapq_thres, keep_improper_pair):

    """
    This function is used for filtering short read with improper pairs,
    and low mapping qualities.
    """

    bamfile_in = pysam.AlignmentFile(input_bam, 'rb')
    bamfile_out = pysam.AlignmentFile(output_bam, 'wb', template = bamfile_in)

    for read in bamfile_in.fetch():
    
        # get the flag information
        flags = format(int(read.flag), "#014b")[:1:-1]

        # skip improper read pair
        if flags[1] != "1" and not keep_improper_pair: continue

        # skip if either of the read pair is unmapped
        if flags[2] == "1" or flags[3] == "1": continue

        # skip supplementary alignment
        if flags[8] == "1" or flags[11] == "1": continue

        # skip duplicated reads
        if flags[10] == "1": continue

        """
        # skip if below the minimum mapping quality
        if (read.mapq < mapq_thres): continue
        """

        bamfile_out.write(read)


def exon_base_count(input_file, output_file, mapq_thres): 

    exon2count = {}
    hIN = open(input_file, 'r')
    hOUT = open(output_file, 'w')

    for line in hIN:
        F = line.rstrip('\n').split('\t')
        if int(F[4]) < mapq_thres: continue
        if int(F[18]) == 0: continue

        exon = '\t'.join(F[12:18])
        if exon in exon2count:
            exon2count[exon] = exon2count[exon] + int(F[18])
        else:
            exon2count[exon] = int(F[18])

    for exon in sorted(exon2count):
        print(exon + '\t' + str(exon2count[exon]), file = hOUT)

    hIN.close()
    hOUT.close()


def mapped_base_count(input_file, output_file, mapq_thres):

    count = 0
    tempID = ""
    with open(input_file, 'r') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            if int(F[4]) < mapq_thres: continue
            if F[3] != tempID:
                if tempID != "":
                    for i in range(len(temp_counts) - 1):
                        count = count + int(temp_counts[i])
                tempID = F[3]
        
            temp_counts = F[10].split(',')
    
    for i in range(len(temp_counts) - 1):
        count = count + int(temp_counts[i])

    hout = open(output_file, 'w')
    print(str(count), file = hout)
    hout.close()



def ref_base_count(input_file, output_file, ref_exon_file):


    ref2len = {}
    with gzip.open(ref_exon_file, 'rt') as hin:
        for line in hin:
            F = line.rstrip('\n').split('\t')
            refID_match = re_refID.search(F[3])
            refID = refID_match.group(1)

            if refID not in ref2len: ref2len[refID] = 0
            ref2len[refID] = ref2len[refID] + int(F[2]) - int(F[1])



    hIN = open(input_file, 'r')
    hOUT = open(output_file, 'w')

    ref2count = {}
    # ref2len = {}

    for line in hIN:
        F = line.rstrip('\n').split('\t')

        refID_match = re_refID.search(F[3])
        refID = refID_match.group(1)
        symbol = re_refID.sub('', F[3])
 
        # refID = F[3]
        # refID = re.sub(r'_\d+$', '', refID)
        # symbol = F[4]
        ID = refID + '\t' + symbol

        if ID not in ref2count: ref2count[ID] = 0
        if ID not in ref2len: ref2len[ID] = 0
             
        ref2count[ID] = ref2count[ID] + int(F[6])
        # ref2len[ID] = ref2len[ID] + int(F[2]) - int(F[1])

    for ID in sorted(ref2count):
        refID, symbol = ID.split('\t')
        print(ID + '\t' + str(ref2len[refID]) + '\t' + str(ref2count[ID]), file = hOUT)
    
    hIN.close()
    hOUT.close()


def sym_base_count(input_file, output_file):

    hIN = open(input_file, 'r')
    hOUT = open(output_file, 'w')

    sym2exp = {}
    for line in hIN:
        F = line.rstrip('\n').split('\t')
        symbol = F[1]

        if symbol not in sym2exp:
            if float(F[2]) > 0.0 and float(F[3]) > 0.0:
                sym2exp[symbol] = F[2] + '\t' + F[3]

        else:

            temp = sym2exp[symbol].split('\t')
            tempV = float(temp[1]) / float(temp[0])

            if float(F[2]) > 0.0 and float(F[3]) > 0.0 and float(F[3]) / float(F[2]) > tempV:
                sym2exp[symbol] = F[2] + '\t' + F[3]

    for symbol in sorted(sym2exp):
        print(symbol + '\t' + str(sym2exp[symbol]), file = hOUT)

    hIN.close() 
    hOUT.close()


def sym_fkpm(input_file, output_file, mapped_base_count_file):

    hIN = open(mapped_base_count_file, 'r')
    mapped_base_count = hIN.readline().rstrip('\n')
    hIN.close()

    hIN = open(input_file, 'r')
    hOUT = open(output_file, 'w')
  
    for line in hIN:
        F = line.rstrip('\n').split('\t')
        fkpm = float(int(F[2]) * 1000 * 1000000) / float(int(mapped_base_count) * int(F[1]))
        print(F[0] + '\t' + str(round(fkpm, 3)), file = hOUT)

    hIN.close()
    hOUT.close()

